@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Image parameter definition */
 
 #ifndef gsiparam_INCLUDED
@@ -20,6 +22,7 @@
 #include "gsccolor.h"		/* for GS_CLIENT_COLOR_MAX_COMPONENTS */
 #include "gsmatrix.h"
 #include "gsstype.h"		/* for extern_st */
+#include "gxbitmap.h"
 
 /* ---------------- Image parameters ---------------- */
 
@@ -297,8 +300,21 @@ void gs_image_t_init_mask_adjust(gs_image_t * pim, bool write_1s,
  * 16 though, due to increased time in image_render_mono_ht in the loop
  * that copies data from scanlines to scancolumns. It seems that writing to
  * the buffer in positions [0] [16] [32] [48] etc is faster than writing in
- * positions [0] [32] [64] [96] etc. For now we leave LAND_BITS set to 16. */
-#define LAND_BITS 16
+ * positions [0] [32] [64] [96] etc. We would therefore like to leave
+ * LAND_BITS set to 16.
+ *
+ * Unfortunately, we call through the device interface to copy_mono with the
+ * results of these buffers, and various copy_mono implementations assume
+ * that the raster given is a multiple of align_bitmap_mod bits. In order to
+ * ensure safely, we therefore set LAND_BITS to max(16, align_bitmap_mod). We
+ * are guaranteed that align_bitmap_mod is a multiple of 16.
+ */
+#define LAND_BITS_MIN 16
+#if LAND_BITS_MIN < (align_bitmap_mod*8)
+#define LAND_BITS (align_bitmap_mod*8)
+#else
+#define LAND_BITS LAND_BITS_MIN
+#endif
 
 /* Used for bookkeeping ht buffer information in landscape mode */
 typedef struct ht_landscape_info_s {
